% Estimating MFPTs

%% Settings

n = 21;  % chain / network size
nidx = 5;  % number of target states to estimate MFPTs to / from
use_parallel = true;  % allow to use as many threads as the number of physical cores
nit = 100000;  % number of steps for the random walk

fprintf('* Chain with %d states.\n', n);
fprintf('* Estimating MFPTs to/from %d states.\n', nidx);
fprintf('* Performing %d iterations.\n', nit);

%% Creating bi-directed chain graph on n nodes
I = eye(n, n);
Self = diag(rand(n, 1));
Next = circshift(eye(n, n), -1, 1);
Prev = circshift(eye(n, n), 1, 1);
W = sparse(Self + (I - Self) / 2 * (Next + Prev));
Wt = W';  % for large networks, may want to keep a single instance of the transpose

%% Selecting which nodes / states to estimate MFPTs to and from
idx = uint64(sort(uint64(randsample(n, nidx, false))));
fprintf('* Target states:\n');
idx'

%% estimating MFPTs
t_mfpt_est = tic;
M_est = mfpt_est(Wt, idx, nit, use_parallel);
fprintf('* MFPTs estimated in %fsec.\n', toc(t_mfpt_est));

%% visualizing
if n > 1000
	error('Visualization for chains of size > 1000 may take a while.');
end

plot(digraph(W));
title('State transition matrix');

figure;
temp = M_est;
temp(temp == 0) = nan;
pcolor(temp);
colormap(1 - autumn);
colorbar();
title(sprintf('$M_{est}, n = %d, n_{it} = %d$', n, nit), 'Interpreter', 'latex');
set(gca, 'FontSize', 14);

t_mfpt_exact = tic;
M_true = mfpt(W);
fprintf('* Computed exact MFPTs in %fsec.\n', toc(t_mfpt_exact));

figure('Position', [ 0.1, 0.1, 1200, 500 ]);
subplot(1, 2, 1);
temp = M_true;
temp(setdiff(1 : n, idx), setdiff(1 : n, idx)) = nan;
diff = abs(M_est - temp);
pcolor(diff);
colormap(1 - autumn);
colorbar();
title(sprintf('$|M_{est} - M_{true}|, n = %d, n_{it} = %d$', n, nit), 'Interpreter', 'latex');
set(gca, 'FontSize', 14);
subplot(1, 2, 2);
temp = M_est;
temp(isnan(temp)) = eps;
spy(temp);
title('$M_{est}$', 'Interpreter', 'latex');
set(gca, 'FontSize', 14);

[i_est, j_est, v_est] = find(M_est);
[i, j, v] = find(M_true);
indic = ismember(i, idx) | ismember(j, idx);
i = i(indic);
j = j(indic);
v = v(indic);
[~, newinds] = sortrows([i, j], 'ascend');
i = i(newinds);
j = j(newinds);
v = v(newinds);
[~, newinds] = sortrows([i_est, j_est], 'ascend');
i_est = i_est(newinds);
j_est = j_est(newinds);
v_est = v_est(newinds);
[v, newinds] = sort(v, 'ascend');
v_est = v_est(newinds);
figure();
plot(v, '.-', 'Color', [ 0.275, 0.510, 0.706 ], 'LineWidth', 1);
hold on;
plot(v_est, '.-', 'Color', [ 0.420, 0.557, 0.137 ], 'LineWidth', 1);
hold on;
nan_indic = isnan(v_est);
v_est(nan_indic) = 0;
v_est(~nan_indic) = nan;
plot(v_est, 'x', 'Color', [ 0.804, 0.361, 0.361 ], 'LineWidth', 1);
legend('exact', 'estimate', 'estimate unavailable',	'Location', 'northwest');
title(sprintf('Sorted MFPTs -- Exact vs. Estimated, $n = %d, n_{it} = %d$', n, nit), 'Interpreter', 'latex');
set(gca, 'FontSize', 14);