% Estimating MFPTs using multiple cores

exp_dir_name = [ armor_fname('ex4_mfpt_est_parallel'), '-', armor_fname(char(datetime)) ];
exp_dir = fullfile(fileparts(mfilename('fullpath')), '@results', exp_dir_name);
ensure_dir_exists_and_writable(exp_dir);
diary(fullfile(exp_dir, 'diary.txt'));


%% Settings

n = 64000;  % chain size
edge_density = 0.05;  % edge density in the uniformly randomly generated state transition network
nidx = 50;  % number of target states to estimate MFPTs to / from
nit = uint64(10^8);  % number of steps for the random walk

fprintf('* Chain with %d states.\n', n);
fprintf('* Estimating MFPTs to/from %d states.\n', nidx);
fprintf('* Numbers of iterations (random walk lengths): %d\n', nit);


%% Creating a Markov chain

t_chain = tic;
fprintf('* Generating a chain... ');
W = sparse([1 0; 0 1]);
while graphconncomp(W) > 1
	W = sprand(n, n, edge_density);
	W = spdiags(1 ./ sum(W, 2), 0, n, n) * W;
end
Wt = W';  % for large networks, may want to keep a single instance of the transpose
fprintf(' DONE in %fsec.\n', toc(t_chain));


%% Selecting which nodes / states to estimate MFPTs to and from
idx = uint64(sort(uint64(randsample(n, nidx, false))));
fprintf('* Target states:\n');
idx'


%% Computing exact MFPTs

fprintf('* Computing exact MFPTs... ');
t_mfpt_exact = tic;
M_exact = mfpt(W, [], Wt);
t_mfpt_exact = toc(t_mfpt_exact);
fprintf('DONE in %fsec.\n', t_mfpt_exact);


%% Estimating MFPTs

maxthreads = uint64(max(1, round(double(feature('numCores')))));
nthreads = [ 1 : maxthreads ]';
t_mfpt_ests = zeros(length(nthreads), 1);
devs = zeros(length(nthreads), 1);
M_ests = cell(length(nthreads), 1);
for i = 1 : length(nthreads)
	use_parallel = nthreads(i);
	fprintf('* Estimating MFPTs using %d threads... ', use_parallel);
	clear mex;
	t_mfpt_est = tic;
	M_ests{i} = mfpt_est(Wt, idx, nit, use_parallel);
	t_mfpt_ests(i) = toc(t_mfpt_est);
	[is, js, vs] = find(M_ests{i});
	nan_indic = isnan(vs);
	is(nan_indic) = [];
	js(nan_indic) = [];
	vs(nan_indic) = [];
	vse = M_exact(sub2ind(size(M_exact), is, js));
	devs(i) = norm(vse - vs, 1) ./ length(vs) / median(vse);
	fprintf('DONE in %fsec (||M_est - M_true||_1 / |MFPT| / median(MFPT) = %.2f).\n', t_mfpt_ests(i), devs(i));
end


%% Visualizing

fprintf('* Visualizing... ');

figure;
plot(1 : length(nthreads), t_mfpt_ests, '*-', 'LineWidth', 1);
hold on;
plot(1 : length(nthreads), double(t_mfpt_ests(1)) ./ double([1 : nthreads(end)]), 'o:k');
legend('actual scaling', 'optimal scaling');
xticks(nthreads);
xlabel('Number of threads', 'Interpreter', 'latex');
ylabel('Time estimating MFPTs, sec', 'Interpreter', 'latex');
title({
	sprintf('Time estimating MFPTs to/from %d states in a chain with %d states', nidx, n),
	sprintf('(exact computation of all MFPTs took %.2fsec)', t_mfpt_exact)
}, 'Interpreter', 'latex');
set(gca, 'FontSize', 14);

figure;
plot(devs, '*-', 'LineWidth', 1);
xticks(nthreads);
xlabel('Number of threads', 'Interpreter', 'latex');
ylabel('Normalized error', 'Interpreter', 'latex');
title('MFPT estimation error', 'Interpreter', 'latex');
set(gca, 'FontSize', 14);

fprintf('DONE.\n');

clear W Wt M_exact;
save(fullfile(exp_dir, 'workspace.mat'));
save_open_figs(exp_dir);
close all;
diary off;