#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>
#include <algorithm>
#include <mutex>
#include "compact_mat.h"
#include "mex_safe.h"
#include "debug.h"


CompactMat::CompactMat(uint64_t n, const uint64_t *nzrc_idx, uint64_t num_nzrc, double fill) {
	_n = n;
	_num_nzrc = num_nzrc;
	_num_zrc = n - num_nzrc;
	
	_nzrc_idx = (uint64_t*)mxMalloc_safe(num_nzrc * sizeof(uint64_t));
	memcpy_safe(_nzrc_idx, nzrc_idx, num_nzrc * sizeof(uint64_t));
	
#ifdef DEBUG
	for (uint64_t ii = 0; ii < num_nzrc; ++ii)
		ASSERT((nzrc_idx[ii] >= 0) && (nzrc_idx[ii] < n), ("Invalid nzrc index: " + std::to_string(nzrc_idx[ii])).c_str());
#endif

	_is_nzrc_ind = (bool*)mxCalloc_safe(n, sizeof(bool));
	for (uint64_t ii = 0; ii < num_nzrc; ++ii) {
		uint64_t i = _nzrc_idx[ii];
		_is_nzrc_ind[i] = true;
	}

	_zrc_idx = (uint64_t*)mxCalloc_safe(_num_zrc, sizeof(uint64_t));
	uint64_t ii = 0;
	for (uint64_t i = 0; i < n; ++i) {
		if (!_is_nzrc_ind[i])
			_zrc_idx[ii++] = i;
	}
	
#ifdef DEBUG
	for (uint64_t ii = 0; ii < _num_zrc; ++ii)
		ASSERT((_zrc_idx[ii] >= 0) && (_zrc_idx[ii] < n), ("Invalid zrc index: " + std::to_string(_zrc_idx[ii])).c_str());
#endif

	_i2ii = (uint64_t*)mxMalloc_safe(_n * sizeof(uint64_t));
	for (uint64_t ii = 0; ii < _num_nzrc; ++ii)
		_i2ii[_nzrc_idx[ii]] = ii;
	for (uint64_t ii = 0; ii < _num_zrc; ++ii)
		_i2ii[_zrc_idx[ii]] = ii;

	_core = (double*)mxMalloc_safe(_num_nzrc * _num_nzrc * sizeof(double));
	std::fill_n(_core, _num_nzrc * _num_nzrc, fill);
	_fat = (double*)mxMalloc_safe(_num_nzrc * _num_zrc * sizeof(double));
	std::fill_n(_fat, _num_nzrc * _num_zrc, fill);
	_tall = (double*)mxMalloc_safe(_num_zrc * _num_nzrc * sizeof(double));
	std::fill_n(_tall, _num_zrc * _num_nzrc, fill);
}


mxArray* CompactMat::to_sparse_mat() const {
	uint64_t nnz = _num_nzrc * (2 * _n - _num_nzrc);
	mxArray *mat = mxCreateSparse_safe(_n, _n, nnz, mxREAL);
	uint64_t *ir = (uint64_t*)mxGetIr_safe(mat);
	uint64_t *jc = (uint64_t*)mxGetJc_safe(mat);
	double *pr = mxGetPr_safe(mat);

	uint64_t irpr_base_ind = 0;
	for (uint64_t j = 0; j < _n; ++j) {
		jc[j] = irpr_base_ind;
		if (_is_nzrc_ind[j]) {
			// sparse column
			for (uint64_t i = 0; i < _n; ++i) {
				double val = get_ij(i, j);
				ir[irpr_base_ind + i] = (_is_nzrc_ind[i] ? _nzrc_idx : _zrc_idx)[_i2ii[i]];
				pr[irpr_base_ind + i] = val;
			}
			irpr_base_ind += _n;
		} else {
			// dense column
			for (uint64_t ii = 0; ii < _num_nzrc; ++ii) {
				double val = get_ij(_nzrc_idx[ii], j);
				ir[irpr_base_ind + ii] = _nzrc_idx[ii];	
				pr[irpr_base_ind + ii] = val;
			}
			irpr_base_ind += _num_nzrc;
		}
	}
	jc[_n] = irpr_base_ind;

	return mat;
}


CompactMat *CompactMat::merge(const std::vector<CompactMat*> cms) {
	if (cms.empty())
		mexErrMsgTxt_safe("CompactMat::merge: attempt to merge an empty list of CompactMat.\n");

	uint64_t n = cms[0]->_n;
	uint64_t num_nzrc = cms[0]->_num_nzrc;
	uint64_t num_zrc = cms[0]->_num_zrc;

#ifdef DEBUG
	for(uint64_t k = 1; k < cms.size(); ++k) {
		ASSERT(cms[0]->_n == cms[k]->_n, "CompactMat::merge: inconsistent _n.");
		ASSERT(cms[0]->_num_nzrc == cms[k]->_num_nzrc, "CompactMat::merge: inconsistent _num_nzrc.");
		ASSERT(cms[0]->_num_zrc == cms[k]->_num_zrc, "CompactMat::merge: inconsistent _num_zrc.");
		ASSERTARREQ(cms[0]->_nzrc_idx, cms[k]->_nzrc_idx, num_nzrc, "CompactMat::merge: inconsistent _nzrc_idx.");
		ASSERTARREQ(cms[0]->_zrc_idx, cms[k]->_zrc_idx, num_zrc, "CompactMat::merge: inconsistent _zrc_idx.");
		ASSERTARREQ(cms[0]->_is_nzrc_ind, cms[k]->_is_nzrc_ind, n, "CompactMat::merge: inconsistent _is_nzrc_ind.");
		ASSERTARREQ(cms[0]->_i2ii, cms[k]->_i2ii, n, "CompactMat::merge: inconsistent _i2ii.");
	}
#endif
	
	uint64_t core_size = num_nzrc * num_nzrc;
	uint64_t fat_tall_size = num_nzrc * num_zrc;
	CompactMat *merged = new CompactMat(n, cms[0]->_nzrc_idx, num_nzrc);
	for (uint64_t k = 0; k < cms.size(); ++k) {
		for (uint64_t lin = 0; lin < core_size; ++lin)
			merged->_core[lin] += cms[k]->_core[lin];
		for (uint64_t lin = 0; lin < fat_tall_size; ++lin) {
			merged->_fat[lin] += cms[k]->_fat[lin];
			merged->_tall[lin] += cms[k]->_tall[lin];
		}
	}
	return merged;
}


#ifdef DEBUG

void CompactMat::print_state() const {
	PRINT(("* n = " + std::to_string(_n) + "\n").c_str());

	PRINT(("* _nzrc_idx (len = " + std::to_string(_num_nzrc) + ") = ").c_str());
	PRINTARRAY(_nzrc_idx, _num_nzrc);
	PRINT("\n");

	PRINT(("* _zrc_idx (len = " + std::to_string(_num_zrc) + ") = ").c_str());
	PRINTARRAY(_zrc_idx, _num_zrc);
	PRINT("\n");

	PRINT(("* _is_nzrc_ind (len = " + std::to_string(_n) + ") = ").c_str());
	PRINTARRAY(_is_nzrc_ind, _n);
	PRINT("\n");

	PRINT(("* _i2ii (len = " + std::to_string(_n) + ") = ").c_str());
	PRINTARRAY(_i2ii, _n);
	PRINT("\n");

	PRINT("* _core:\n");
	PRINTMAT(_core, _num_nzrc, _num_nzrc);

	PRINT("* _fat:\n");
	PRINTMAT(_fat, _num_nzrc, _num_zrc);

	PRINT("* _tall:\n");
	PRINTMAT(_tall, _num_zrc, _num_nzrc);
}

#endif // DEBUG